@@ -1,11 +1,17 @@
 use strict;
 use warnings;
+use Carp;
+
 package Software::LicenseUtils;
-BEGIN {
-  $Software::LicenseUtils::VERSION = '0.103002';
+{
+  $Software::LicenseUtils::VERSION = '0.103008';
 }
 # ABSTRACT: little useful bits of code for licensey things
 
+use File::Spec;
+use IO::Dir;
+use Module::Load;
+
 
 my $_v = qr/(?:v(?:er(?:sion|\.))(?: |\.)?)/i;
 my @phrases = (
@@ -32,26 +38,62 @@ my @phrases = (
   'MIT'                        => 'MIT',
 );
 
+my %meta_keys = ();
+
+# find all known Software::License::* modules and get identification data
+#
+# XXX: Grepping over @INC is dangerous, as it means that someone can change the
+# behavior of your code by installing a new library that you don't load.  rjbs
+# is not a fan.  On the other hand, it will solve a real problem.  One better
+# solution is to check "core" licenses first, then fall back, and to skip (but
+# warn about) bogus libraries.  Another is, at least when testing S-L itself,
+# to only scan lib/ blib. -- rjbs, 2013-10-20
+for my $lib (map { "$_/Software/License" } @INC) {
+  next unless -d $lib;
+  for my $file (IO::Dir->new($lib)->read) {
+    next unless $file =~ m{\.pm$};
+
+    # if it fails, ignore it
+    eval {
+      (my $mod = $file) =~ s{\.pm$}{};
+      my $class = "Software::License::$mod";
+      load $class;
+      $meta_keys{ $class->meta_name }{$mod}  = undef;
+      $meta_keys{ $class->meta2_name }{$mod} = undef;
+      my $name = $class->name;
+      unshift @phrases, qr/\Q$name\E/, [$mod];
+    };
+  }
+}
+
 sub guess_license_from_pod {
   my ($class, $pm_text) = @_;
   die "can't call guess_license_* in scalar context" unless wantarray;
+  return unless $pm_text =~ /
+    (
+      =head \d \s+
+      (?:licen[cs]e|licensing|copyright|legal)\b
+    )
+  /ixmsg;
+
+  my $header = $1;
 
 	if (
 		$pm_text =~ m/
+      \G
       (
-        =head \d \s+
-        (?:licen[cs]e|licensing|copyright|legal)\b
         .*?
       )
       (=head\\d.*|=cut.*|)
       \z
     /ixms
   ) {
-		my $license_text = $1;
+		my $license_text = "$header$1";
 
     for (my $i = 0; $i < @phrases; $i += 2) {
       my ($pattern, $license) = @phrases[ $i .. $i+1 ];
-			$pattern =~ s{\s+}{\\s+}g;
+			$pattern =~ s{\s+}{\\s+}g
+				unless ref $pattern eq 'Regexp';
 			if ( $license_text =~ /$pattern/i ) {
         my $match = $1;
 				# if ( $osi and $license_text =~ /All rights reserved/i ) {
@@ -70,35 +112,53 @@ sub guess_license_from_pod {
 	return;
 }
 
-my %meta_keys = (
-  perl         => 'Perl_5',
-  apache       => [ map { "Apache_$_" } qw(1_1 2_0) ],
-  artistic     => 'Artistic_1_0',
-  artistic_2   => 'Artistic_2_0',
-  lgpl         => [ map { "LGPL_$_" } qw(2_1 3_0) ],
-  bsd          => 'BSD',
-  gpl          => [ map { "GPL_$_" } qw(1 2 3) ],
-  mit          => 'MIT',
-  mozilla      => [ map { "Mozilla_$_" } qw(1_0 1_1) ],
-);
-
 
 sub guess_license_from_meta {
   my ($class, $meta_text) = @_;
   die "can't call guess_license_* in scalar context" unless wantarray;
 
-  my ($license_text) = $meta_text =~ m{\b["']?license["']?\s*:\s*["']?([a-z_]+)["']?}gm;
+  my ($license_text) = $meta_text =~ m{\b["']?license["']?\s*:\s*["']?([a-z_0-9]+)["']?}gm;
 
   return unless $license_text and my $license = $meta_keys{ $license_text };
 
-  return map { "Software::License::$_" } ref $license ? @$license : $license;
+  return map { "Software::License::$_" } sort keys %$license;
 }
 
 *guess_license_from_meta_yml = \&guess_license_from_meta;
 
+my %short_name = (
+  'GPL-1'      =>  'Software::License::GPL_1',
+  'GPL-2'      =>  'Software::License::GPL_2',
+  'GPL-3'      =>  'Software::License::GPL_3',
+  'LGPL-2'     =>  'Software::License::LGPL_2',
+  'LGPL-2.1'   =>  'Software::License::LGPL_2_1',
+  'LGPL-3'     =>  'Software::License::LGPL_3_0',
+  'LGPL-3.0'   =>  'Software::License::LGPL_3_0',
+  'Artistic'   =>  'Software::License::Artistic_1_0',
+  'Artistic-1' =>  'Software::License::Artistic_1_0',
+  'Artistic-2' =>  'Software::License::Artistic_2_0',
+);
+
+
+sub new_from_short_name {
+  my ( $class, $arg ) = @_;
+
+  Carp::croak "no license short name specified"
+    unless defined $arg->{short_name};
+  my $short = delete $arg->{short_name};
+  Carp::croak "Unknow license with short name $short"
+    unless $short_name{$short};
+
+  my $lic_file = my $lic_class = $short_name{$short} ;
+  $lic_file =~ s!::!/!g;
+  require "$lic_file.pm";
+  return $lic_class->new( $arg );
+}
+
 1;
 
 __END__
+
 =pod
 
 =head1 NAME
@@ -107,7 +167,7 @@ Software::LicenseUtils - little useful bits of code for licensey things
 
 =head1 VERSION
 
-version 0.103002
+version 0.103008
 
 =head1 METHODS
 
@@ -129,16 +189,26 @@ Given the content of the META.(yml|json) file found in a CPAN distribution, this
 method makes a guess as to which licenses may apply to the distribution.  It
 will return a list of zero or more Software::License instances or classes.
 
+=head2 new_from_short_name
+
+  my $license_object = Software::LicenseUtils->new_from_short_name( {
+     short_name => 'GPL-1',
+     holder => 'X. Ample'
+  }) ;
+
+Create a new L<Software::License> object from the license specified
+with C<short_name>. Known short license names are C<GPL-*>, C<LGPL-*> ,
+C<Artistic> and C<Artistic-*>
+
 =head1 AUTHOR
 
 Ricardo Signes <rjbs@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 by Ricardo Signes.
+This software is copyright (c) 2013 by Ricardo Signes.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
